c
c     ----------------------------------------------------------------
c
      subroutine remdc(x,npts,n1,n2)
c  
c     remove the dc (calculated between points n1 and n2) from the 
c     array x
c
      include 'zplot.par'
      real x(npmax)
c
      xsum=0.
c
      do 10 i=n1,n2
         xsum=xsum+x(i)
10    continue
c
      if(xsum.ne.0.) then
        dc=xsum/float(n2-n1+1)
      else
        return
      end if
c
      do 20 i=1,npts
         x(i)=x(i)-dc
20    continue
c
      return
      end 
c
c     ----------------------------------------------------------------
c
      subroutine crscor(pilot,ncrcor,x,npts,n0,nlag,lag)
c  
c     determine optimal lag of array x with respect to pilot trace
c     using simple cross-correlation method
c
      include 'zplot.par'
      real pilot(npmax),x(npmax)
c
      ccmax=-1.e20
c 
      do 10 i=-nlag,nlag,1
c
         ccsum=0.
c
         ntlag=n0+i-1
         do 20 j=1,ncrcor
            ccsum=ccsum+pilot(j)*x(ntlag+j)
20       continue
c
         if(ccsum.gt.ccmax) then
           lag=i
           ccmax=ccsum
         end if
10    continue
c
      return
      end
c
c     ----------------------------------------------------------------
c
      subroutine mensym(x,y,albht,label,icol,ifbcol)
c  
c     plot the name of a menu item
c
      character*14 label
c
      if(icol.lt.2) then
ccc        call rtxcol(ifbcol,ifbcol)
        call pcolor(ifbcol)
      else
ccc        call rtxcol(icol,icol)
        call pcolor(icol)
      end if
c
      call symbol(x,y,albht,label,0.,14)
c
      return
      end
c
c     ----------------------------------------------------------------
c
      subroutine mennum(x,xshift,y,albht,value,ndeci,icol,ifbcol)
c  
c     plot the value of a menu item
c
      if(icol.lt.2) then
ccc        call rtxcol(ifbcol,ifbcol)
        call pcolor(ifbcol)
      else
ccc        call rtxcol(icol,icol)
        call pcolor(icol)
      end if
c
      call number(x+xshift*albht,y,albht,value,0.,ndeci)
c
      return
      end
c        
c     ----------------------------------------------------------------
c        
      subroutine sort(ra,rb,n)
c
c     sort the elements of array x in order of increasing size using
c     a heapsort technique
c
      real ra(n)
      integer rb(n)
c
      do 30 i=1,n
         rb(i)=i
30    continue
c
      l=n/2+1
      ir=n
c
10    continue
c
      if(l.gt.1) then
        l=l-1     
        rra=ra(l)
        rrb=rb(l)
      else
        rra=ra(ir)
        rrb=rb(ir)
        ra(ir)=ra(1)
        rb(ir)=rb(1)
        ir=ir-1
        if(ir.eq.1) then
          ra(1)=rra
          rb(1)=rrb
          return
        end if
      end if
      i=l
      j=l+l
20    if(j.le.ir) then
        if(j.lt.ir) then
          if(ra(j).lt.ra(j+1)) j=j+1
        end if
        if(rra.lt.ra(j)) then
          ra(i)=ra(j)
          rb(i)=rb(j)
          i=j
          j=j+j
        else
          j=ir+1
        end if
        go to 20
      end if
      ra(i)=rra
      rb(i)=rrb
      go to 10
c
      end
c            
c     ----------------------------------------------------------------
c            
      subroutine pick(seis,npts,nstart,nfini,tmin,time,nwind,sints,
     +                iflag,minenratio)
c            
c     make an arrival pick of a trace in seis by calculating the
c     ratio of the energy in a window on either side
c     of each point and selecting the point corresponding to the
c     largest ratio
c            
      include 'zplot.par'
c
      real seis(npts),seis2(npmax),max,minenratio
c
      iflag=0
      nw2=nwind*2
      nit=min(npts-nw2,nfini-nw2)
      nstpnw=nstart+nwind
      if(nit-nstart.lt.2) return
      do 5 i=1,npts
         seis2(i)=seis(i)*seis(i)
5     continue
      max=minenratio
c
      sum1=0.
      sum2=0.
      do 20 j=1,nwind
         sum1=sum1+seis2(nstpnw-j)
         sum2=sum2+seis2(nstpnw+j)
20    continue
      er=sum2/sum1
      if(er.gt.max) then
        ipos=nstart
        max=er
        iflag=1
      end if
c
      do 10 i=nstart+1,nit
         im1=i-1
         sum1=sum1-seis2(im1)+seis2(im1+nwind)
         sum2=sum2-seis2(i+nwind)+seis2(i+nw2)
         er=sum2/sum1
         if(er.gt.max) then
           ipos=i
           max=er
           iflag=1
         end if
10    continue
c
      if(iflag.eq.1) time=tmin+float(ipos+nwind-1)*sints
c
      return
      end
c
c     ----------------------------------------------------------------
c
      subroutine vg2(x,npts,sints,tvg,pvg)
c
c     vg (variable gain) filter
c
c     apply vg to trace x by dividing each point by the sum of the
c     absolute value of the points raised to the pvg power in a
c     window of length tvg centred about that point
c
c     x ------- array containing trace to be filtered
c     npts ---- number of points in array x 
c     sints --- sampling interval in seconds
c     tvg ----- filter window length in seconds
c     pvg ----- power of filter:   pvg<0  --  accentuate 'highs'
c                                  pvg=0  --  has no effect
c                                  pvg>0  --  bring up 'lows'
c
      include 'zplot.par'
      real*4 x(npts),e(npmax),xvg(npmax)
c
      nw=nint(tvg/sints)+1
      if(mod(nw,2).eq.0) nw=nw+1
      n1=nw/2+1
      if(nw.lt.1) nw=1
      if(nw.gt.npts) nw=npts
c
      do 10 i=1,npts
         e(i)=abs(x(i))**pvg
         xvg(i)=0.
10    continue
c
      esum=0.
      do 20 i=1,nw
         esum=esum+e(i)
20    continue
c
      do 30 i=1,n1
         if(esum.ne.0.) xvg(i)=x(i)/esum
30    continue
c
      if(nw.lt.npts) then
        do 40 i=n1+1,npts-n1
           esum=esum-e(i-n1)+e(i+n1-1)
           if(esum.ne.0.) xvg(i)=x(i)/esum
40      continue
        esum=esum-e(npts-nw)+e(npts)
      end if
c
      do 50 i=npts-n1+1,npts
         if(esum.ne.0.) xvg(i)=x(i)/esum
50    continue
c
      do 60 i=1,npts
         x(i)=xvg(i)
60    continue
c
      return
      end
c
c     ----------------------------------------------------------------
c
      subroutine template(iopentp,tpfile,xtemp,ttemp,ntemp,rdepth,
     +                    iflagm,iwater,vwater,sdepth)
c
c     read in the pick template file
c
      include 'zplot.par'
      real xtemp(nptemp),ttemp(nptemp)
      character tpfile*80
c
      iflagm=0
c
100   open(18, file=tpfile, status='old', err=99, iostat=ipfile)
99    if(ipfile.gt.0.or.tpfile.eq.'') then
        write(6,5)
5       format(/'***  there is no pick template file  ***')
        write(*,*) char(7)
        write(6,15)
15      format('enter name of pick template file: ')
        read(5,25) tpfile
25      format(a80)
        if(tpfile.eq.'') then
          iflagm=1
          return
        end if
        go to 100
      end if
c
      iopentp=1
      i=1
200   read(18,*,end=300) xtemp(i),ttemp(i)
      i=i+1
      go to 200
300   ntemp=i-1
      if(ntemp.le.1) then
        write(*,*) char(7)
        write(6,35)
35      format(/
     +  '***  pick template must have at least 2 points  ***'/)
        iflagm=1 
        return
      end if
c
      if(iwater.eq.1) then
        depth=rdepth-sdepth
        do 10 i=1,ntemp
           h=(depth**2+xtemp(i)**2)**.5
           twater=h/vwater
           if(twater.lt.ttemp(i)) ttemp(i)=twater
10      continue
      end if
c
      return
      end
